@@ -35,7 +35,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: kalog.c,v 1.4.2.1 2002/07/11 14:00:41 lha Exp $");
+RCSID("$Id: kalog.c,v 1.8 2003/01/17 03:30:21 lha Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -50,7 +50,11 @@ RCSID("$Id: kalog.c,v 1.4.2.1 2002/07/11 14:00:41 lha Exp $");
 #include <arlalib.h>
 
 #include <ka.cs.h>
+#ifdef HAVE_OPENSSL
+#include <openssl/des.h>
+#else
 #include <des.h>
+#endif
 #include <krb.h>
 
 #include <ka-procs.h>
@@ -58,6 +62,7 @@ RCSID("$Id: kalog.c,v 1.4.2.1 2002/07/11 14:00:41 lha Exp $");
 #include <roken.h>
 #include <err.h>
 
+#include <vers.h>
 
 static void
 parse_user (char *argv1, const char **user, const char **cell)
@@ -88,7 +93,8 @@ main (int argc, char **argv)
     const char *cellname;
     const char *user;
 
-    setprogname (argv[0]);
+    set_progname (argv[0]);
+    tzset();
 
     method = log_open (getprogname(), "/dev/stderr:notime");
     if (method == NULL)
@@ -100,10 +106,13 @@ main (int argc, char **argv)
 
     if (argc == 1)
 	user = get_default_username();
-    else if (argc == 2)
+    else if (strcmp("-version", argv[1]) == 0) {
+	print_version(NULL);
+	return 0;
+    } else if (argc == 2)
 	parse_user (argv[1], &user, &cellname);
     else {
-	fprintf (stderr, "usage: %s [username[@cell]]\n",
+	fprintf (stderr, "usage: %s [-version] [username[@cell]]\n",
 		 getprogname());
 	exit (1);
     }
