@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2000, 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: strsep_copy.c,v 1.1 2000/08/16 01:23:49 lha Exp $");
+RCSID("$Id: strsep_copy.c,v 1.3 2002/09/12 16:23:32 lha Exp $");
 #endif
 
 #include <string.h>
@@ -53,8 +53,10 @@ strsep_copy(const char **stringp, const char *delim, char *buf, size_t len)
 	return -1;
     *stringp = *stringp + strcspn(*stringp, delim);
     l = min(len, *stringp - save);
-    memcpy(buf, save, l);
-    buf[l] = '\0';
+    if(len > 0) {
+	memcpy(buf, save, l);
+	buf[l] = '\0';
+    }
 
     l = *stringp - save;
     if(**stringp == '\0')
