@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 2000 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -56,7 +56,7 @@
 #include <roken.h>
 #include <kafs.h>
 
-RCSID("$Id: fhbench.c,v 1.6.2.2 2002/07/11 14:00:59 lha Exp $");
+RCSID("$Id: fhbench.c,v 1.10 2002/09/07 10:49:45 lha Exp $");
 
 struct fhb_handle {
     char data[512];
@@ -156,7 +156,7 @@ nop_call (void)
 	vice_ioctl.out      = NULL;
 	vice_ioctl.out_size = 0;
 	
-	ret = k_pioctl (NULL, VIOC_XFSDEBUG, &vice_ioctl, 0);
+	ret = k_pioctl (NULL, VIOC_NNPFSDEBUG, &vice_ioctl, 0);
 	if (ret < 0)
 	    err (1, "k_pioctl");
     }
@@ -341,7 +341,9 @@ main (int argc, char **argv)
     if (write_file < 0)
 	usage (1);
 
+#ifdef KERBEROS
     if (!k_hasafs())
+#endif
 	errx (1, "no afs kernel module");
 
     handles = emalloc (num_files * sizeof(*handles));
