@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,11 +31,13 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: lex.h,v 1.4 1999/03/01 08:38:22 assar Exp $ */
+/* $Id: lex.h,v 1.6 2000/10/02 22:36:33 lha Exp $ */
 
 #ifndef _LEX_
 #define _LEX_
 
 void error_message (int errorp, char *, ...);
 
+int yylex (void);
+
 #endif /* _LEX_ */
